import numpy as np
from lmfit import Minimizer, Parameters, fit_report
import time
#nacteni dat do matice
data=np.loadtxt('Data.dat') 
xdata=data[:,0]
ydata=data[:,1]

#definice residualu: rozdilu fitovane funkce a dat
def residual(params, x, y):
	#rozbaleni hodnot parametru ze slovniku
	eINF=params['eINF']
	w=params['w']
	g=params['g']
	e = eINF-w**2/(x*(x+1j*g))    # dielektricka funkce
	N = e**0.5              # index lomu
	R = abs((N-1)/(N+1))**2-y      # odrazivost
	return R

params = Parameters()#definice objektu s parametry pro model
#zde se definuji jmena parametru, jejich startovaci hodnoty a zda se fituji ci nikoliv
params.add('eINF', value=9,vary=True)
params.add('w', value=1780,vary=True)
params.add('g', value=300,vary=True)
original=ydata+residual(params,xdata,ydata) # Modelove spektrum se startovnimi parametry

TimeStart=time.time()
minner = Minimizer(residual, params, fcn_args=(xdata,ydata)) #fitovani
result = minner.minimize()  #zde jsou vysledky
final = ydata + result.residual  # calculate final result: data + residual
TimeEnd=time.time()    
print ("Cas minimalizace: ", TimeEnd-TimeStart , " s")


print(fit_report(result)) # write error report, alternative result.params.pretty_print()

#ulozeni vysledneho fitu do souboru
FileData= open('Spektrum.dat', 'w+')
for i in range(len(xdata)):
    print (xdata[i],"\t",final[i], file=FileData)
FileData.close()

#ulozeni statistiky do souboru
FileFit = open('Statistika.dat', 'w+')
print(fit_report(result),file=FileFit)
FileFit.close()

ResultingP= result.params.valuesdict() #Toto jsou vysledne hodnoty parametru, pro pozdejsi pouziti
#print(ResultingP)

# vykresleni fitu na monitor
import pylab
pylab.plot(xdata, ydata, 'k-', label='Data')
pylab.plot(xdata, final, 'r-',label='Nafitovany model')
pylab.plot(xdata, original, 'b-',label='Puvodni model')
pylab.legend(loc='upper right', shadow=False)
pylab.show()


